import importlib
import sys


def import_path(path):
    """
    Imports any valid python module or attribute path as though it were a
    module

    :Example:
        >>> from yamlconf import import_path
        >>> from my_package.my_module.my_submodule import attribute
        >>> attribute.sub_attribute == \
        ...     import_path("y_package.my_module.my_submodule.attribute.sub_attribute")
        True
        >>>

    :Parameters:
        path : `str`
            A valid python path that crosses modules and/or attributes
    """  # noqa
    sys.path.insert(0, ".")
    parts = path.split(".")
    module = None

    # Import the module as deeply as possible.  Prioritize an attribute chain
    # over a module chain
    for i in range(1, len(parts)+1):
        if module is not None and hasattr(module, parts[i-1]):
            try:
                return _import_attributes(module, parts[i-1:])
            except AttributeError:
                pass
        module_path = ".".join(parts[0:i])
        module = importlib.import_module(module_path)

    return module


def _import_attributes(module, attributes):
    for attribute in attributes:
        module = getattr(module, attribute)

    return module



from .import_module import import_module
from .import_path import import_path
from .load import load, loads
from .merge import merge
from .propagate_defaults import propagate_defaults

__all__ = [import_module, import_path, load, loads, merge, propagate_defaults]

__version__ = "0.2.3"



from nose.tools import eq_

from ..merge import merge


def test_merge():
    a = {
        'foo': {
            'bar': {
                'foo': 5,
                'bar': 7
            }
        },
        'l': [1, 5, 'foo']
    }
    b = {
        'foo': {
            'bar': {
                'foo': 6
            }
        },
        'bar': 5,
        's': "I'm a string",
        'l': [1, 2, 3]
    }
    c = {
        'foo': {
            'bar': {
                'foobar': 10
            }
        }
    }
    expected = {
        'foo': {
            'bar': {
                'foo': 6,
                'bar': 7,
                'foobar': 10
            }
        },
        'bar': 5,
        's': "I'm a string",
        'l': [1, 2, 3]
    }
    eq_(merge(a, b, c),
        expected)




def merge(d, *dicts):
    """
    Recursively merges dictionaries
    """

    for d_update in dicts:
        if not isinstance(d, dict):
            raise TypeError("{0} is not a dict".format(d))

        dict_merge_pair(d, d_update)

    return d


def dict_merge_pair(d1, d2):
    """
    Recursively merges values from d2 into d1.
    """
    for key in d2:
        if key in d1 and isinstance(d1[key], dict) and \
           isinstance(d2[key], dict):
            dict_merge_pair(d1[key], d2[key])
        else:
            d1[key] = d2[key]

    return d1



import os

from setuptools import find_packages, setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


def requirements(fname):
    for line in open(os.path.join(os.path.dirname(__file__), fname)):
        yield line.strip()


setup(
    name="yamlconf",
    version="0.2.3",  # Update in yamlconf/__init__.py too
    author="Aaron Halfaker",
    author_email="ahalfaker@wikimedia.org",
    description=("This library provides a means to read yaml configuration " +
                 "files and propagate default values in reasonable ways.  " +
                 "Nothing complicated."),
    license="MIT",
    url="https://github.com/halfak/yamlconf",
    packages=find_packages(),
    long_description=read('README.rst'),
    install_requires=['pyyaml'],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
)
